//
//  MKBeaconCB.m
//  Beacon-CB
//
//  Created by Matthias Krauß on 15.09.14.
//  Copyright (c) 2014 Matthias Krauß. All rights reserved.
//

#import "MKBeaconCB.h"

#if TARGET_OS_IPHONE
#import <CoreBluetooth/CoreBluetooth.h>
#else
#import <IOBluetooth/IOBluetooth.h>
#endif


@interface MKBeaconCB () <CBCentralManagerDelegate>

@property (strong) CBCentralManager* central;

@end

@implementation MKBeaconCB

- (void) startScan {
    self.central = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
}

- (void)centralManagerDidUpdateState:(CBCentralManager *)central {
    CBCentralManagerState state = self.central.state;
    switch (state) {
        case CBCentralManagerStatePoweredOff:
            NSLog(@"Sorry, Bluetooth is off");
            exit(0);
            break;
        case CBCentralManagerStateUnsupported:
            NSLog(@"Sorry, no BLE");
            exit(0);
            break;
        case CBCentralManagerStateUnauthorized:
            NSLog(@"Sorry, not authorized");
            exit(0);
            break;
        case CBCentralManagerStatePoweredOn:
            NSLog(@"BLE is on, starting scan");
            [self.central scanForPeripheralsWithServices:@[] options:@{}];
            break;
        default:
            NSLog(@"Central changed state to %i",(int)state);
            break;
    }
}

- (void)centralManager:(CBCentralManager*)central didDiscoverPeripheral:(CBPeripheral*)peripheral advertisementData:(NSDictionary*)advertisementData RSSI:(NSNumber*)RSSI {
    NSString* name = [peripheral name];
    if (!name) name = @"(unnamed)";
    NSString* advKeys = [[advertisementData allKeys] componentsJoinedByString:@","];
    if ([advKeys length] < 1) advKeys = @"(none)";
    NSLog(@"discovered device %@ RSSI %i advertised: %@",name,[RSSI intValue],advKeys);
    [self checkBeaconAdvertisement:advertisementData];
}

- (BOOL) checkBeaconAdvertisement:(NSDictionary*)advDict {
    NSData* manufData = [advDict objectForKey:@"kCBAdvDataManufacturerData"];
    if (!manufData) return NO;    
    if ([manufData length] != 25) return NO;
    const uint8_t* data = [manufData bytes];
    
    if ((data[0] != 0x4c) || (data[1] != 0x00)) return NO;  //Not Apple
    if (data[2] != 0x02) return NO;                         //No iBeacon advertisement
    if (data[3] != 0x15) return NO;                         //Payload length wrong
    
    NSMutableString* beaconId = [NSMutableString string];
    for (int i=4; i<20; i++) {
        if (i>4) [beaconId appendString:@"-"];
        [beaconId appendFormat:@"%02x",data[i]];
    }
    
    uint16_t major = (data[20] << 8) | data[21];
    uint16_t minor = (data[22] << 8) | data[23];
    int refRSSI = data[24]-256;
    
    NSLog(@"Found beacon with ID %@ Major %i Minor %i RefRSSI %i",beaconId, major, minor, refRSSI);

    return YES;
}

@end
