//
//  ViewController.m
//  ThermoStat-HK
//
//  Created by Matthias Krauß on 24.09.14.
//  Copyright (c) 2014 Matthias Krauß. All rights reserved.
//

#import "ViewController.h"
#import <HealthKit/HealthKit.h>
#import "MKBarGraphView.h"

#define UPDATE_SECONDS 1.0
#define BIN_SECONDS 5.0
#define BIN_COUNT 10

@interface ViewController ()

@property (strong) NSTimer* timer;
@property (strong) HKHealthStore* healthStore;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupHealthKit];
    self.barView.minVal = 35.0;
    self.barView.maxVal = 42.0;
    // Do any additional setup after loading the view, typically from a nib.
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void) viewWillAppear:(BOOL)animated {
    SEL selector = @selector(triggerUpdate);
    NSMethodSignature* signature = [self methodSignatureForSelector:selector];
    NSInvocation* invocation = [NSInvocation invocationWithMethodSignature:signature];
    [invocation setTarget:self];
    [invocation setSelector:selector];
    self.timer = [NSTimer scheduledTimerWithTimeInterval:UPDATE_SECONDS invocation:invocation repeats:YES];
}

- (void) viewDidDisappear:(BOOL)animated {
    [self.timer invalidate];
    self.timer = nil;
}


- (void) setupHealthKit {
    if (![HKHealthStore isHealthDataAvailable]) {
        NSLog(@"Sorry, no Health Kit");
        exit(0);
    }
    self.healthStore = [HKHealthStore new];
    NSSet* shareSet = [NSSet set];
    HKQuantityType* tempType = [HKQuantityType quantityTypeForIdentifier:HKQuantityTypeIdentifierBodyTemperature];
    NSSet* readSet = [NSSet setWithObjects:tempType, nil];
    [self.healthStore requestAuthorizationToShareTypes:shareSet readTypes:readSet completion:
        ^(BOOL ok, NSError* err) {
            //Nothing to do, we didn't get an honest answer anyway
        }
     ];
}

- (void) triggerUpdate {
    
    HKQuantityType* tempType = [HKQuantityType quantityTypeForIdentifier:HKQuantityTypeIdentifierBodyTemperature];
    NSDate* now = [NSDate date];
    NSDate* start = [now dateByAddingTimeInterval:-(BIN_SECONDS*BIN_COUNT)];
    NSPredicate* predicate = [HKQuery predicateForSamplesWithStartDate:start endDate:now options:HKQueryOptionNone];
    
    NSDateComponents* interval = [NSDateComponents new];
    interval.second = BIN_SECONDS;
    
    HKStatisticsCollectionQuery* query = [[HKStatisticsCollectionQuery alloc]
                                          initWithQuantityType:tempType
                                          quantitySamplePredicate:predicate
                                          options:HKStatisticsOptionDiscreteAverage
                                          anchorDate:now
                                          intervalComponents:interval];
    query.initialResultsHandler = ^(HKStatisticsCollectionQuery* query, HKStatisticsCollection* result, NSError* error) {
        if (error) {
            NSLog(@"Query error: %@",error.localizedDescription);
        } else {
            [self performSelectorOnMainThread:@selector(handleQueryResult:)
                                   withObject:result
                                waitUntilDone:NO];
        }
    };
    [self.healthStore executeQuery:query];
}

- (void) handleQueryResult:(HKStatisticsCollection*)results {
    NSDate* end = [NSDate date];
    NSDate* start = [end dateByAddingTimeInterval:-(BIN_SECONDS*BIN_COUNT)];
    NSMutableArray* values = [NSMutableArray array];
    HKUnit* celsiusUnit = [HKUnit unitFromString:@"degC"];
    [results enumerateStatisticsFromDate:start toDate:end withBlock:^(HKStatistics* bin, BOOL* stop) {
        HKQuantity* quantity = [bin averageQuantity];
        double value = quantity ? [quantity doubleValueForUnit:celsiusUnit] : 0.0;
        [values addObject:[NSNumber numberWithDouble:value]];
        *stop = (values.count >= BIN_COUNT);
    }];
    self.barView.values = values;
    [self.barView setNeedsDisplay];
}


@end
